﻿// ----- TextButton Object
// ----- Code by tedGo, based on a sample by T.P. Wang

function TextButton(text, func, x, y, w, h, size_options, options, colours, tiptext, funcOption) {
	this.text = text;

	this.func = func;

	this.x = x;
	this.y = y;
	this.w = w;
	this.h = h;

	var tmp_size = ["text_x_margin", "text_y_margin", "func_left_pad", "func_top_pad", "func_right_pad", "func_bottom_pad"];
	for (var l = 0; l < 6; l++) this[tmp_size[l]] = size_options && size_options[tmp_size[l]] ? size_options[tmp_size[l]] : 0;
	this.font_size = size_options && size_options.font_size ? size_options.font_size : 12;

	this.left = x + this.func_left_pad;
	this.top = y + this.func_top_pad;
	this.w_ = w - this.func_left_pad - this.func_right_pad;
	this.h_ = h - this.func_top_pad - this.func_bottom_pad;
	this.right = x + w - this.func_right_pad;
	this.bottom = y + h - this.func_bottom_pad;

	var as_tmp = Math.min(this.w_, this.h_) / 2;
	this.arc_size = size_options && size_options.arc_size ? Math.min(as_tmp, size_options.arc_size) : Math.min(as_tmp, Math.min(this.w, this.h) / 6);

	var tmp_opt = ["btn_style", "font_style", "line_width", "text_align_h", "text_align_v", "text_onclick_shift", "text_shadow", "btn_depth"];
	for (var i = 0; i < 8; i++) this[tmp_opt[i]] = options && options[tmp_opt[i]] ? options[tmp_opt[i]] : 0;
	this.font_name = options && options.font_name ? options.font_name : "";
	this.text_shift = 0;

	var tmp_col = ["shadow_colour", "text_normal", "line_normal", "back_normal", "text_hover", "line_hover", "back_hover", "text_down", "line_down", "back_down"];
	for (var j = 0; j < 10; j++) this[tmp_col[j]] = colours && colours[tmp_col[j]] ? colours[tmp_col[j]] : j < 4 ? null : this[tmp_col[j - 3]];
	this.colour_text = this[tmp_col[1]];
	this.colour_line = this[tmp_col[2]];
	this.colour_back = this[tmp_col[3]];

	this.tiptext = tiptext ? tiptext : "";

	this.funcOption = funcOption;

	this.state = ButtonStates.normal;

	this.traceMouse = function(x, y) {
		if (this.state == ButtonStates.hide) return false;

		var b = this.left < x && x < this.right && this.top < y && y < this.bottom;

		if (b) btn_down ? this.changeState(ButtonStates.down) : this.changeState(ButtonStates.hover);
		else this.changeState(ButtonStates.normal);
		return b;
	}

	this.changeState = function(newstate) {
		newstate != this.state && this.repaint();
		this.state = newstate;

		this.text_shift = this.state == ButtonStates.down ? this.text_onclick_shift : 0;

		switch (this.state) {
			case ButtonStates.normal:
				this.colour_text = this.text_normal;
				this.colour_line = this.line_normal;
				this.colour_back = this.back_normal;
				break;

			case ButtonStates.hover:
				this.colour_text = this.text_hover;
				this.colour_line = this.line_hover;
				this.colour_back = this.back_hover;
				break;

			case ButtonStates.down:
				this.colour_text = this.text_down;
				this.colour_line = this.line_down;
				this.colour_back = this.back_down;
				break;

			default:
				this.colour_text = this.colour_line = this.colour_back = null;
		}
	}

	this.draw = function(gr) {
		if (this.btn_style == 1) {
			this.colour_back && gr.FillSolidRect(this.left, this.top, this.w_, this.h_, this.colour_back);
			this.btn_depth > 0 && gr.DrawRect(this.left, this.top + this.btn_depth, this.w_, this.h_ - this.btn_depth, this.btn_depth, 0x1FFFFFFF);
			this.btn_depth > 0 && gr.DrawRect(this.left, this.top, this.w_, this.h_ - this.btn_depth, this.btn_depth, 0x3F000000);
			this.colour_line && this.line_width > 0 && gr.DrawRect(this.left, this.top, this.w_, this.h_, this.line_width, this.colour_line);
		}

		if (this.btn_style == 2) {
			this.colour_back && gr.FillRoundRect(this.left, this.top, this.w_, this.h_, this.arc_size, this.arc_size, this.colour_back);
			this.btn_depth > 0 && gr.DrawRoundRect(this.left, this.top + this.btn_depth, this.w_, this.h_ - this.btn_depth, this.arc_size, this.arc_size, this.btn_depth, 0x1FFFFFFF);
			this.btn_depth > 0 && gr.DrawRoundRect(this.left, this.top, this.w_, this.h_ - this.btn_depth, this.arc_size, this.arc_size, this.btn_depth, 0x3F000000);
			this.colour_line && this.line_width > 0 && gr.DrawRoundRect(this.left, this.top, this.w_, this.h_, this.arc_size, this.arc_size, this.line_width, this.colour_line);
		}

		if (this.btn_style == 3) {
			this.colour_back && gr.FillEllipse(this.left, this.top, this.w_, this.h_, this.colour_back);
			this.btn_depth > 0 && gr.DrawEllipse(this.left, this.top + this.btn_depth, this.w_, this.h_ - this.btn_depth, this.btn_depth, 0x1FFFFFFF);
			this.btn_depth > 0 && gr.DrawEllipse(this.left, this.top, this.w_, this.h_ - this.btn_depth, this.btn_depth, 0x3F000000);
			this.colour_line && this.line_width > 0 && gr.DrawEllipse(this.left, this.top, this.w_, this.h_, this.line_width, this.colour_line);
		}

		var txt_h = this.text != "" ? gr.CalcTextHeight(this.text, gdi.Font(this.font_name, this.font_size, this.font_style)) : 0;
		var txt_y = this.text_align_v == 1 ? this.y + Math.floor((this.h - txt_h) / 2) : this.text_align_v == 2 ? this.y + Math.floor(this.h / 3 * 2) : this.y;
		txt_h > 0 && this.text_shadow > 0 && gr.GdiDrawText(this.text, gdi.Font(this.font_name, this.font_size, this.font_style), this.shadow_colour, this.x + this.text_x_margin + Math.abs(this.text_shift) + this.text_shadow, txt_y + this.text_y_margin + this.text_shift + this.text_shadow, this.w, txt_h, this.text_align_h);
		txt_h > 0 && gr.GdiDrawText(this.text, gdi.Font(this.font_name, this.font_size, this.font_style), this.colour_text, this.x + this.text_x_margin + Math.abs(this.text_shift), txt_y + this.text_y_margin + this.text_shift, this.w, txt_h, this.text_align_h);
	}

	this.repaint = function() {
		window.RepaintRect(this.x, this.y, this.w, this.h);
	}

	this.onClick = function() {
		this.func && this.func();
	}

	this.onMouseIn = function() {
		btn_tooltip = window.CreateTooltip();
		btn_tooltip.Text = this.tiptext;
		btn_tooltip.Activate();
	}

	this.onMouseOut = function() {
		btn_tooltip.Deactivate();
		btn_tooltip.Dispose();
	}
}